from zeep import xsd


class MutatieRegel:
    def __init__(
        self,
        bedrag_invoer: str,
        bedrag_excl_btw: str,
        bedrag_btw: str,
        bedrag_incl_btw: str,
        btw_code: str,
        btw_percentage: str,
        tegenrekening_code: str,
    ):
        self.bedrag_invoer = bedrag_invoer
        self.bedrag_excl_btw = bedrag_excl_btw
        self.bedrag_btw = bedrag_btw
        self.bedrag_incl_btw = bedrag_incl_btw
        self.btw_code = btw_code
        self.btw_percentage = btw_percentage
        self.tegenrekening_code = tegenrekening_code

    def export(self):
        return dict(
            BedragInvoer=self.bedrag_invoer,
            BedragExclBTW=self.bedrag_excl_btw,
            BedragBTW=self.bedrag_btw,
            BedragInclBTW=self.bedrag_incl_btw,
            BTWCode=self.btw_code,
            BTWPercentage=self.btw_percentage,
            TegenrekeningCode=self.tegenrekening_code,
            KostenplaatsID=xsd.SkipValue,
        )



from zeep import Client as ZeepClient
from .models import Mutatie


class Client:
    client = None
    session_id: str = None

    username: str = None
    security_code_1: str = None
    security_code_2: str = None

    def __init__(self, username: str, security_code_1: str, security_code_2: str):
        self.username = username
        self.security_code_1 = security_code_1
        self.security_code_2 = security_code_2
        self.client = ZeepClient("https://soap.e-boekhouden.nl/soap.asmx?wsdl")
        self.connect()

    def connect(self):
        session = self.client.service.OpenSession(
            Username=self.username, SecurityCode1=self.security_code_1, SecurityCode2=self.security_code_2
        )
        self.session_id = session["SessionID"]

    def add_mutatie(self, mutatie: Mutatie):
        exported_mutatie = mutatie.export()
        params = dict(SessionID=self.session_id, SecurityCode2=self.security_code_2, oMut=exported_mutatie)
        response = self.client.service.AddMutatie(**params)
        return response



import pytest
import os

from eboekhoudenapi import Client, Mutatie, MutatieRegel

USERNAME = os.environ["USERNAME"]
SECURITY_CODE_1 = os.environ["SECURITY_CODE_1"]
SECURITY_CODE_2 = os.environ["SECURITY_CODE_2"]


def get_client():
    return Client(
        username=USERNAME,
        security_code_1=SECURITY_CODE_1,
        security_code_2=SECURITY_CODE_2,
    )


def get_mutatie():
    amount_excl = 20
    amount_incl = 20 / 100 * 121
    vat = amount_excl / 100 * 21

    mutatie_regels = [
        MutatieRegel(
            bedrag_invoer=str(amount_incl),
            bedrag_excl_btw=str(amount_excl),
            bedrag_btw=str(vat),
            bedrag_incl_btw=str(amount_incl),
            btw_code="HOOG_VERK_21",
            btw_percentage="21",
            tegenrekening_code="8002",
        ),
        MutatieRegel(
            bedrag_invoer=str(amount_incl * 2),
            bedrag_excl_btw=str(amount_excl * 2),
            bedrag_btw=str(vat * 2),
            bedrag_incl_btw=str(amount_incl * 2),
            btw_code="HOOG_VERK_21",
            btw_percentage="21",
            tegenrekening_code="8003",
        ),
    ]

    return Mutatie(
        soort="FactuurVerstuurd",
        datum="2021-01-01",
        rekening="1300",
        relatie_code="036",
        factuurnummer="test-1231241246",
        omschrijving="Automatisch import via api",
        betalingstermijn="14",
        inexbtw="IN",
        mutatie_regels=mutatie_regels,
    )


def test_client():
    get_client()


def test_mutatie_export():
    mutatie = get_mutatie()

    result = mutatie.export()


def test_mutatie():
    mutatie = get_mutatie()
    client = get_client()

    res = client.add_mutatie(mutatie=mutatie)
    print(res)



from .mutatie_regel import MutatieRegel


class Mutatie:
    def __init__(
        self,
        soort: str,
        datum: str,
        rekening: str,
        relatie_code: str,
        factuurnummer: str,
        omschrijving: str,
        betalingstermijn: str,
        inexbtw: str,
        mutatie_regels: [MutatieRegel],
    ):
        self.soort = soort
        self.datum = datum
        self.rekening = rekening
        self.relatie_code = relatie_code
        self.factuurnummer = factuurnummer
        self.omschrijving = omschrijving
        self.betalingstermijn = betalingstermijn
        self.inexbtw = inexbtw
        self.mutatie_regels = mutatie_regels

    def export(self):
        exported_mutatie_regels = []

        for mutatie_regel in self.mutatie_regels:
            exported_mutatie_regels.append(mutatie_regel.export())

        return dict(
            MutatieNr="99999",
            Soort=self.soort,
            Datum=self.datum,
            Rekening=self.rekening,
            RelatieCode=self.relatie_code,
            Factuurnummer=self.factuurnummer,
            Omschrijving=self.omschrijving,
            Betalingstermijn=self.betalingstermijn,
            InExBTW=self.inexbtw,
            MutatieRegels=dict(cMutatieRegel=exported_mutatie_regels),
        )
